import numpy as np
from apollo_py_numpy_core.spatial.quaternions import UnitQuaternion
from apollo_py_numpy_core.spatial.rotation_matrices import Rotation3
from apollo_py_numpy_core.spatial.vectors import V3


class Isometry3:
    def __init__(self, q: UnitQuaternion, v: V3):
        self.rotation = q
        self.translation = v

    def map_point(self, v: V3) -> 'V3':
        return self.rotation.map_point(v) + self.translation

    def __matmul__(self, other: 'Isometry3') -> 'Isometry3':
        new_rotation = self.rotation @ other.rotation
        new_translation = self.rotation.map_point(other.translation) + self.translation
        return Isometry3(new_rotation, new_translation)

    def __mul__(self, other: 'Isometry3') -> 'Isometry3':
        return self@other

    def __repr__(self) -> str:
        return f"Isometry3(\n  rotation: {np.array2string(self.rotation)},\n  ----- \n  translation: {np.array2string(self.translation)}\n)"

    def __str__(self) -> str:
        return f"Isometry3(\n  rotation: {np.array2string(self.rotation)},\n  ----- \n  translation: {np.array2string(self.translation)}\n)"


class IsometryMatrix3:
    def __init__(self, r: Rotation3, v: V3):
        self.rotation = r
        self.translation = v

    def map_point(self, v: V3) -> 'V3':
        return self.rotation.map_point(v) + self.translation

    def __matmul__(self, other: 'IsometryMatrix3') -> 'IsometryMatrix3':
        new_rotation = self.rotation @ other.rotation
        new_translation = (self.rotation @ other.translation + self.translation).view(V3)
        return IsometryMatrix3(new_rotation, new_translation)

    def __mul__(self, other: 'IsometryMatrix3') -> 'IsometryMatrix3':
        return self@other

    def __repr__(self) -> str:
        return f"IsometryMatrix3(\n  rotation: \n{np.array2string(self.rotation)},\n  ----- \n  translation: {np.array2string(self.translation)}\n)"

    def __str__(self) -> str:
        return f"IsometryMatrix3(\n  rotation: \n{np.array2string(self.rotation)},\n  ----- \n  translation: {np.array2string(self.translation)}\n)"
